{+--------------------------------------------------------------------------+
 | Class:       TIDEStream4
 | Created:     8.98
 | Author:      Martin Waldenburg (Add Comment and UnComment By QingHai)
 | Copyright    1997, 1998 all rights reserved.
 | Description: A simple but effective interface to the IDE's text buffer of D4
 |              You can retrive the text of the IDE's text buffer either
 |              as PChar or as string list.
 |              You can manipulate it in any way and then replace the
 |              whole text with the result of your manipulation.
 |              This D4 version needs mwota.zip.
 | Version:     1.0
 | Status:      FreeWare
 | Disclaimer:
 | This is provided as is, expressly without a warranty of any kind.
 | You use it at your own risc.
 +--------------------------------------------------------------------------+}
unit mwIDEStream4;

interface

uses
  Windows,
  SysUtils,
  Messages,
  Classes,
  ToolsApi,
  mwota;

type
  TIDEStream4 = class(TMemoryStream)
  private
    fLines: TStringList;
    function GetAsPChar: PChar;
    function GetStreamTextLen: Longint;
  protected
  public
    procedure WriteText(Text: PChar);
    property Capacity;
    property AsPChar: PChar read GetAsPChar;
    procedure AddComment;
    procedure UnComment;
    function GetText: PChar;
    procedure SetMemPointer(Ptr: Pointer; Size: Longint);
    procedure GetLines(newValue: TStringList);
    property StreamTextLen: Longint read GetStreamTextLen;
  published
  end;

var
  ActualReader: IOTAEditReader;
  ActualWriter: IOTAEditWriter;

implementation

function strMake(C: Char; Len: Integer): String;
begin
  Result:='';
  while Length(Result)<Len do Result:=C+Result;
end;


function LeftStr(const S: String; Len: Integer): string;
begin
  Result:=Copy(S,1,Len);
end;

function RightStr(const S: String; Len: Integer): string;
begin
  if Len>=Length(S) then
    Result:=S
  else
    Result:=Copy(S,Succ(Length(S))-Len,Len);
end;

function TIDEStream4.GetAsPChar: PChar;
const
  TheEnd: Char = #0;
begin
  Position := Size;
  Write(TheEnd, 1);
  SetPointer(Memory, Size - 1);
  Result := Memory;
end; { GetAsPChar }

procedure TIDEStream4.SetMemPointer(Ptr: Pointer; Size: Longint);
begin
  SetPointer(Ptr, Size);
end; { SetMemPointer }

function TIDEStream4.GetStreamTextLen: Longint;
const
  BuffLen = 16383;
var
  TextBuffer: PChar;
  Readed, BuffPos, TextLen: LongInt;
begin
  TextLen := 0;
  GetMem(TextBuffer, BuffLen + 1);
  BuffPos := 0;
  ActualReader:= TOTA.CreateReader;
  try
    repeat
      Readed := ActualReader.GetText(BuffPos, TextBuffer, Bufflen);
      TextLen := TextLen + Readed;
      inc(BuffPos, Readed);
    until Readed < BuffLen;
  finally
    FreeMem(TextBuffer, BuffLen + 1);
    ActualReader:= nil;
  end;
  Result := TextLen;
end; { GetStreamTextLen }

function TIDEStream4.GetText: PChar;
const
  BuffLen = 16383;
var
  TextBuffer: PChar;
  Readed, BuffPos: LongInt;
begin
  Clear;
  GetMem(TextBuffer, BuffLen + 1);
  BuffPos := 0;
  ActualReader:= TOTA.CreateReader;
  try
    repeat
      Readed := ActualReader.GetText(BuffPos, TextBuffer, Bufflen);
      Write(TextBuffer^, Readed);
      inc(BuffPos, Readed);
    until Readed < BuffLen;
  finally
    FreeMem(TextBuffer, BuffLen + 1);
    ActualReader:= nil;
  end;
  Result := AsPchar;
end; { GetText }

procedure TIDEStream4.WriteText(Text: PChar);
var
  TextLen: LongInt;
begin
  TextLen := GetStreamTextLen;
  ActualWriter:= TOTA.CreateWriter;
  try
    ActualWriter.CopyTo(0);
    ActualWriter.DeleteTo(TextLen - 1);
    ActualWriter.Insert(Text);
  finally
    ActualWriter:= nil;
  end;
end; { WriteText }

procedure TIDEStream4.GetLines(newValue: TStringList);
begin
  FLines := NewValue;
  if assigned(FLines) then
  begin
    FLines.Clear;
    Flines.SetText(GetText);
  end;
end; { GetLines }

procedure TIDEStream4.AddComment;
var
  EditorI:IOTASourceEditor;//ǰ༭SourceEditor
  sLine,eLine:integer;//BlockĿʼкͽ
  i:integer;
begin
  EditorI:=TOTA.GetSourceEditor;
  sLine:=EditorI.BlockStart.Line;
  eLine:=EditorI.BlockAfter.Line;
  if Assigned(FLines) then
  for i :=sLine-1  to eLine-2 do
  begin
    FLines[i]:='//'+FLines[i];
  end;
  EditorI:=nil;
end;

procedure TIDEStream4.UnComment;
var
  EditorI:IOTASourceEditor;//ǰ༭SourceEditor
  sLine,eLine:integer;//BlockĿʼкͽ
  i:integer;
  s1,s2:String;
begin
  EditorI:=TOTA.GetSourceEditor;
  sLine:=EditorI.BlockStart.Line;
  eLine:=EditorI.BlockAfter.Line;
  if Assigned(FLines) then
  for i :=sLine-1  to eLine-2 do
  begin
    s1:=TrimLeft(FLines[i]);
    if LeftStr(s1,2)='//' then
    begin
      s2:=StrMake(' ',Length(FLines[i])-Length(S1));//ոɵִ
      FLines[i]:=S2+RightStr(S1,Length(S1)-2);
    end;
  end;
  EditorI:=nil;
end;

end.




























